//
// Benzaiten - A Simple Game Framework.
// Copyright (C) 2008, 2009 Geisha Studios.
//
// This library is free software;  you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation; either version 2.1 of the License, or
// (at your option) any later version.
//
// This library is distributed in hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
// License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this library; if not, write to the Free Software Foundation, Inc.,
// 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
//
#if defined (HAVE_BENZAITEN_CONFIG_H)
#include <benzaiten_config.h>
#endif // !HAVE_BENZAITEN_CONFIG_H
#include "Game.hpp"
#include <cassert>
#include <string>
#include <boost/bind.hpp>
#include "EventManager.hpp"
#include "FrameRateManager.hpp"
#include "GameStateManager.hpp"
#include "Surface.hpp"

using namespace benzaiten;

Game::Game ():
    over_ (false)
{
}

void
Game::onQuit ()
{
    over (true);
}

inline bool
Game::over () const
{
    return over_;
}

inline void
Game::over (bool isOver)
{
    over_ = isOver;
}

void
Game::run (EventManager &eventManager, GameStateManager &stateManager,
        Surface &screen)
{
    // Connect the Quit signal so we know when the window closed.
    eventManager.OnQuit.connect (boost::bind (&Game::onQuit, this));

    // Go for a frame rate of 35FPS.
    FrameRateManager frameRateManager (35);

    while ( !over () )
    {
        frameRateManager.update ();
        eventManager.update ();

        if ( stateManager.hasActiveState () )
        {
            IGameState &activeState (stateManager.activeState (eventManager));
            activeState.update (frameRateManager.elapsedTime ());
            activeState.draw (screen);

            screen.flip ();
        }
        else
        {
            // If there's no active state, then we are done and
            // can quit the game.
            over (true);
        }
    }
}
