//
// Benzaiten - A Simple Game Framework.
// Copyright (C) 2008, 2009 Geisha Studios.
//
// This library is free software;  you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation; either version 2.1 of the License, or
// (at your option) any later version.
//
// This library is distributed in hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
// License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this library; if not, write to the Free Software Foundation, Inc.,
// 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
//
#if !defined (GEISHA_STUDIOS_BENZAITEN_MUSIC_HPP)
#define GEISHA_STUDIOS_BENZAITEN_MUSIC_HPP

#if defined (_MSC_VER) && (_MSC_VER >= 1020)
#pragma once
#endif // _MSC_VER && _MSC_VER >= 1020

#include <SDL_mixer.h>
#include <boost/shared_ptr.hpp>

namespace benzaiten
{
    ///
    /// @class Music
    /// @brief A music.
    ///
    class Music
    {
        public:
            /// A pointer to a music.
            typedef boost::shared_ptr<Music> ptr;

            /// An infinite number of times to play.
            enum
            {
                Infinite = -1
            };

            ///
            /// @brief Constructor.
            ///
            /// @param[in] music The smart pointer to the music to hold.
            ///            It must be a valid pointer.
            ///
            Music (boost::shared_ptr<Mix_Music> music);

            ///
            /// @brief Plays the music the specified number of times.
            ///
            /// @param[in] times The number of times to play the music.
            ///            \c Infinite is an special constant that
            ///            will play the music until stopped.
            ///
            void play (int times = Infinite) const;

        private:
            /// The real music object.
            boost::shared_ptr<Mix_Music> music_;
    };
}

#endif // !GEISHA_STUDIOS_BENZAITEN_MUSIC_HPP
