//
// Benzaiten - A Simple Game Framework.
// Copyright (C) 2008, 2009 Geisha Studios.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
#if !defined (GEISHA_STUDIOS_BENZAITEN_SOUND_HPP)
#define GEISHA_STUDIOS_BENZAITEN_SOUND_HPP

#if defined (_MSC_VER) && (_MSC_VER >= 1020)
#pragma once
#endif // _MSC_VER && _MSC_VER >= 1020

#include <SDL_mixer.h>
#include <boost/shared_ptr.hpp>

namespace benzaiten
{
    ///
    /// @class Sound
    /// @brief A sound effect.
    ///
    class Sound
    {
        public:
            /// A pointer to a sound.
            typedef boost::shared_ptr<Sound> ptr;

            ///
            /// @brief Constructor.
            ///
            /// @param[in] sound The smart pointer to the sound effect to
            ///            hold.  It must be a valid pointer.
            ///
            Sound (boost::shared_ptr<Mix_Chunk> sound);

            ///
            /// @brief Plays the sound on the first free channel.
            ///
            void play () const;

        private:
            /// The real sound effect.
            boost::shared_ptr<Mix_Chunk> sound_;
    };
}

#endif // !GEISHA_STUDIOS_BENZAITEN_SOUND_HPP
